<?php
/*
Plugin Name: MudSlideShow
Plugin URI: http://www.sebaxtian.com/acerca-de/mudslideshow
Description: An image gallery system using PicasaWeb.
Version: 0.9.1
Author: Sebaxtian
Author URI: http://www.sebaxtian.com
*/

/*  Copyright 2007-2009  Sebaxtian  (email : sebaxtian@gawab.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

define('RANDOM_PHOTO', -1);
define('LAST_PHOTO', -2);
define('NO_USER', false);
define('MUDS_MNMX_V', 0.3);

define('MUDS_WIDGET_RANDOM',1);
define('MUDS_WIDGET_LAST',2);
define('MUDS_WIDGET_RANDOMLAST',3);
define('MUDS_WIDGET_RANDOMFROM',4);

$muds_type=get_option('muds_gallery_type');
$muds_columns=get_option('muds_columns');
$muds_rows=get_option('muds_rows');
$muds_user=get_option('muds_user');

//Set the viewer functions
switch($muds_type) {
	case '1':
		require('viewers/highslide-viewer.php');
		break;
	case '2':
		require('viewers/lytebox-viewer.php');
		break;
	default:
		require('viewers/none-viewer.php');
		break;
}

add_action('init', 'muds_add_buttons');
add_action('init', 'muds_text_domain');
add_action('admin_menu', 'muds_menus');
add_action('wp_head', 'muds_header');
add_action('activate_mudslideshow/mudslideshow.php', 'muds_activate');
add_action('wp_footer', 'muds_footer');
add_filter('the_content', 'muds_content');

require_once('sources/picasa.php');

/**
* To declare where are the mo files (i18n).
* This function should be called by an action.
*
* @access public
*/
function muds_text_domain() {
	load_plugin_textdomain('mudslide', 'wp-content/plugins/mudslideshow/lang');
}

/**
* Function to return the url of the plugin concatenated to a string. The idea is to
* use this function to get the entire URL for some file inside the plugin.
*
* @access public
* @param string str The string to concatenate
* @return The URL of the plugin concatenated with the string 
*/
function muds_plugin_url($str = '')
{

	$aux = '/wp-content/plugins/mudslideshow/'.$str;
	$aux = str_replace('//', '/', $aux);
	$url = get_bloginfo('wpurl');
	return $url.$aux;

}

/**
* Function to return the path to the cache concatenated to a string. The idea is to
* use this function to get the entire path for some cache file.
*
* @access public
* @param string str The string to concatenate
* @return The path of the plugin concatenated with the string 
*/
function muds_plugin_cache($str = '')
{
	$ans = parse_url(get_bloginfo('wpurl'));
	$aux = $_SERVER['DOCUMENT_ROOT'].$ans['path'].'/wp-content/cache/mudslideshow/'.$str;
	$aux = str_replace('//', '/', $aux);
	return $aux;

}

/**
* Function to create the database and to add options into WordPress
* This function should be called by an action.
*
* @access public
*/
function muds_activate()
{
	global $wpdb;
	global $db_version;

	//The default options
	add_option('muds_gallery_type', 0);
	add_option('muds_columns', 5);
	add_option('muds_rows', 0);
	add_option('muds_user', '');
	
	$table_name = $wpdb->prefix . "mudslide";
	if($wpdb->get_var("show tables like '$table_name'") != $table_name)
	{

		$sql = "CREATE TABLE $table_name(
			id bigint(1) NOT NULL AUTO_INCREMENT,
			type int NOT NULL,
			file text NOT NULL,
			name text NOT NULL,
			PRIMARY KEY (id)
			);";

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		dbDelta($sql);
		add_option('muds_db_version', 1);
	}
	
	//Create the cache directory if it doesn't exist
	$ans = parse_url(get_bloginfo('wpurl'));
	$cache_dir = $_SERVER['DOCUMENT_ROOT'].$ans['path'].'/wp-content/cache';
	
	if(!file_exists($cache_dir)) mkdir($cache_dir);
	if(!file_exists($cache_dir.'/mudslideshow')) mkdir($cache_dir.'/mudslideshow');
	
}

/**
* Enable menus.
* This function should be called by an action.
*
* @access public
*/
function muds_menus() {
	add_management_page('MudSlideShow', 'MudSlideShow', 10, 'mudsmanage', 'muds_manage_page');
	add_options_page('MudSlideShow', 'MudSlideShow', 10, 'mudsoptions', 'muds_options');
}


/**
* Return the file of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The filename of the gallery. False if it doesn't exists.
*/
function muds_get_gallery_file($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT file FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Return the type of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The gallery type. False if it doesn't exists.
*/
function muds_get_gallery_type($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT type FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Return the name of a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return string The gallery name. False if it doesn't exists.
*/
function muds_get_gallery_name($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_var("SELECT name FROM $table_name WHERE id=$id")) {
		$answer=$aux;
	}
	return $answer;
}

/**
* Deletes a gallery from the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return bool True on succes. False if not.
*/
function muds_delete_gallery($id) {
	global $wpdb;
	$answer=false;
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_results("SELECT type, file FROM $table_name WHERE id=$id")) {
		$type=$aux[0]->type;
		$filename=$aux[0]->file;
		$del = @unlink(muds_plugin_cache($filename));
		$db = $wpdb->query("DELETE FROM $table_name WHERE id=$id");
		if($del && $db) $answer = true;
	}
	return $answer; 
}

/**
* Updates a gallery in the internal list.
*
* @access public
* @param int id ID of the gallery in the internal list.
* @return bool True on succes. False if not.
*/
function muds_update_gallery($id) {
	global $wpdb;
	$answer=false;
	
	//Get the data from this gallery
	$table_name = $wpdb->prefix . "mudslide";
	if($aux = $wpdb->get_results("SELECT type, file FROM $table_name WHERE id=$id")) {
		$type=$aux[0]->type;
		$filename=$aux[0]->file;
		
		switch($type) {
			case MUDS_PICASA:	//If this is a picasa album
				$answer = mudsPicasa_update_gallery_by_filename($filename);
				break;
		}
	}
	return $answer;
}

/**
* Page to manage the galleries likst.
*
* @access public
*/
function muds_manage_page()
{
	global $muds_type;
	global $wpdb;
	$table_name = $wpdb->prefix . "mudslide";
	$messages=array();
	if(!function_exists('minimax_version') || minimax_version()<MUDS_MNMX_V) { //Do we have minimax?
		array_push($messages, sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V ));
	}

	$mode_x=$_POST['mode_x']; //Something to execute?
	$mode=$_GET['mode']; //Something to show?
	
	$doaction=false; //CLicked the button for bulk action?
	if($_POST['doaction']!="") $doaction=$_POST['action']; //Top button
	if($_POST['doaction2']!="") $doaction=$_POST['action2']; //Bottom button
	if($doaction) {
		switch($doaction) {
			case 'delete': //Delete som galleries
				if(count($_POST['checked_galleries'])>0) {
					foreach($_POST['checked_galleries'] as $checked_id) { //Delete each gallery
						muds_delete_gallery($checked_id);
					}
				}
				break;
		}
	}
	
	switch($mode) { //Something to do?
		case 'delete': //Delete one gallery
			check_admin_referer('mudslide_deletegallery');
			$id=$_GET['id'];
			//If deleted, show the message too.
			if(muds_delete_gallery($id)) array_push($messages, __("Gallery deleted",'mudslide'));
			break;
	}
	
	//Do we have to show a message?
	if(count($messages)>0) {
		echo "<div class='updated'>";
		foreach($messages as $message) echo "<p><strong>$message</strong></p>";
		echo "</div>";
	}

	// Now display the options editing screen
	// options form
	include('templates/mudslide_manage.php');
}

/**
* Page to manage the options.
*
* @access public
*/
function muds_options()
{
	global $muds_type, $muds_columns, $muds_rows, $wpdb;
	$messages=array();

	$mode_x=$_POST['mode_x']; //Someting to execute?
	$mode=$_GET['mode']; //Something to show?
	
	switch($mode_x) {
		case 'manage_x': //Update the config data
			$mode='manage';
			
			$muds_type = $_POST['muds_gallery_type'];
			$muds_columns= $_POST['muds_columns'];
			$muds_rows= $_POST['muds_rows'];
			
			// Save the posted value in the database
			update_option( 'muds_gallery_type', $muds_type );
			update_option( 'muds_columns', $muds_columns);
			update_option( 'muds_rows', $muds_rows);

			// Put an 'options updated' message on the screen
			array_push($messages,__( 'Options saved', 'mudslide' ));
			
			break;
	}
	
	switch($mode) {
		case 'manage':
			break;
	}
	
	//Do we have messages to show?
	if(count($messages)>0) {
		echo "<div class='updated'>";
		foreach($messages as $message) echo "<p><strong>$message</strong></p>";
		echo "</div>";
	}
	
	//Ok, show the dialog
	include('templates/mudslide_options.php');
}

/**
* Filter to manage contents. Check for [mudslide] tags.
* This function should be called by a filter.
*
* @access public
* @param string content The content to change.
* @return string The content with the changes the plugin have to do.
*/
function muds_content($content) {

	//Picasa - Gallery
	$search = "@(?:<p>)*\s*\[mudslide:picasa,(\d+),([\w.]+),(\d+|[\w#]+)(,\d+,\d+|,\d+,(left|right|center)|)?\]\s*(?:</p>)*@i";
	if(preg_match_all($search, $content, $matches)) { //search for mudslide picasa
		if (is_array($matches)) { //We found
			foreach ($matches[1] as $key =>$v0) { //for each gallery, do
				//Get the values
				$search = $matches[0][$key];
				$conf=$matches[1][$key];
				$user=$matches[2][$key];
				$gallery=$matches[3][$key];
				$last=$matches[4][$key];
			
				$begin=0;
				$end=0;
				$simple=false;
				
				//If we have data at the end, search if it is for the begin and the end or
				//for a simple gallery 
				if($last) {
					$data=explode(',', $last);
					if(is_numeric($data[2])) { //This is begin-end data
						$begin=$data[1];
						$end=$data[2];
						if($first>$last) {
							$first=$last=0;
						}
					} else { //Simple gallery
						$simple=true;
						$size=$data[1];
						$float=$data[2];
					}
					
				}
				
				if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
					if(!$simple) { //If it is not a ismple gallery, show the gallery, else show simple
						$replace = mudsPicasa_show_gallery( $user, $gallery, $begin, $end, $conf );
					} else {
						$replace = mudsPicasa_show_simple($user, $gallery, $size, $float, $conf);
					}
				} else { //If we don't have minimax, ask for it to the user.
					$replace = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
				}
				//Ok, done with this, go ahead 
				$content= str_replace ($search, $replace, $content);
			}
		}
	}

	//Picasa - Photo
	$search = "@(?:<p>)*\s*\[mudslide:picasa,(\d+),([\w.]+),(\d+|[\w#]+),(\d+),(\d+),(left|right|center)?\]\s*(?:</p>)*@i";
	if(preg_match_all($search, $content, $matches)) { //search for picasa photo
		if (is_array($matches)) { //we found
			foreach ($matches[1] as $key =>$v0) { //foreach photo, do
				//get the values
				$search = $matches[0][$key];
				$conf=$matches[1][$key];
				$user=$matches[2][$key];
				$gallery=$matches[3][$key];
				$photo=$matches[4][$key];
				$size=$matches[5][$key];
				$float=$matches[6][$key];

				if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
					$replace = mudsPicasa_show_photo( $user, $gallery, $photo, $size, $float, $conf ); //Show the photo
				} else {
					//If we don't have minimax, ask for it to the user.
					$replace = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
				}
				$content = str_replace ($search, $replace, $content);
			}
		}
	}

	return $content;
}

/**
* Create the HTML code for a simple gallery. If first and last are equal to 0, show all the gallery
*
* @access public
* @param xml data The data description of the gallery.
* @param string update_post The post elements for the data.php script to update this gallery.
* @param int first The first foto to show
* @param int first The last foto to show
* @param int conf A configuration number for future releases.
* @return string The HTML code for the simple frame viewer.
*/
function muds_gallery($data, $update_post, $first=0, $last=0, $conf) {
	global $muds_type, $muds_columns, $post, $current_user; //The configuration data to show the gallery
	
	$rand = mt_rand(111111,999999); //The identifier
	$answer="";
	$count=1; //Count the number of the picture
	$column=1; //Counter to determine if we have to jump
	$table="";
	
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		$answer="";
		if($first==0 || $last==0) { //If we don't have begin and end, show all the gallery
			$first=1;
			$last=count($photo_gallery->photo);
		}
		
		foreach($photo_gallery->photo as $photo) { //Go trhough the gallery
			if($count>=$first && $count<=$last) { //If the photo is between the first and the last picture to show
				if($column==1) { //Are we starting a row?
					$table.="<tr>"; //We start the row
				}
				
				//Get the data
				$media = str_replace('%size%', 800, $photo->resize);
				$thumbnail = str_replace('%size%', 72, $photo->resize);
				$title=$photo->title;
				$caption = false;
				if($conf & 1) { //Should we show the comment
					$caption = $photo->comment;
				}
				$table.="<td align='center'><span class='slide'>".muds_show($media, $thumbnail, $title, $rand, 'center', $caption, 72)."</span></td>"; //Create a cell with the photo
				if($column==$muds_columns) { //is this the last picture from the row
					$table.="</tr>"; //close the row
					$column=0; //Start the counter again
				}

				$column++; //Add 1 to the column counter
			}
			$count++; //Add 1 to the picture counter
		}
		
		//No more pictures
		if($column!=1) { //Did we left a row opened?
			$table.="</tr>"; //Close the opened row
		}
		
		//Section to update the gallery
		if($post->post_author == $current_user->id || current_user_can('edit_others_posts')) { //Can this user edit the post
			//Add a minimax function to update the gallery
			$table.="<tr><td colspan='$muds_columns' align='right'>
				<div id='throbber-msspage$rand' class='throbber-off'><small><a style='cursor : pointer;' id='update-$rand' onclick=\"
					var update_gallery$rand = function() {
						if (mss$rand.xhr.readyState == 4 && mss$rand.xhr.status==200 ) {
							window.location.reload();
						}
					}
					document.getElementById('update-$rand').innerHTML='".__('Updating', 'mudslide')."';
					var mss$rand=new minimax('".muds_plugin_url('/ajax/update.php')."', false);
					mss$rand.setThrobber('throbber-msspage$rand', 'throbber-on', 'throbber-off');
					mss$rand.setFunc(update_gallery$rand);
					mss$rand.post('$update_post');\">".__('Update','mudslide')."</a></small></div></td></tr>";
		}
		
		//We have owr table!!!!!
		$answer = "<div><table class='mudslideshow'>$table</table></div>";	
	}
	
	return $answer;
}

/**
* Create the HTML code for a simple gallery
*
* @access public
* @param xml data The data description of the gallery.
* @param string update_post The post elements for the data.php script to update this gallery.
* @param int size The size of the frame.
* @param string float The side where the frame has to float.
* @param int conf A configuration number for future releases.
* @return string The HTML code for the simple frame viewer.
*/
function muds_simple($data, $update_post, $size, $float, $conf) {
	global $post, $current_user;
	
	//Calculate the position for the buttons
	$box_height=$size+10;
	$box_width=$size+10;
	$slidemeter_left=$size-110;
	$slidemeter_top=$size-35;
	
	$id=rand(1000000,9999999);
	$throbber="<div id='slidemeter$id' class='slidemeter'></div>";
		
	//Section to update the gallery
	if($post->post_author == $current_user->id || current_user_can('edit_others_posts')) { //can this user edit the post
		//Add a minimax function to update the gallery
		$throbber="<div id='throbber-msspage$rand' class='throbbersimp-off' onclick=\"var update_gallery$rand = function() {
			if (mss$rand.xhr.readyState == 4 && mss$rand.xhr.status==200 ) {
				window.location.reload();
			}
		}
		var mss$rand=new minimax('".muds_plugin_url('/ajax/update.php')."', false);
		mss$rand.setThrobber('throbber-msspage$rand', 'throbbersimp-on', 'throbbersimp-off');
		mss$rand.setFunc(update_gallery$rand);
		mss$rand.post('$update_post');\" />$throbber</div>
		";
	}
	
	//Data to align the frame
	if($float=='center') {
		$center=" align=\"center\"";
	}
	$class="";
	if($float) {
		$class=" class=\"$align\"";
	}	
	
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		//Create the simple frame	
		$answer="<div id='simple_gallery$id' style='background : url(".muds_plugin_url('/img/simpleslide/background.png').") no-repeat center center; border-style : solid; border-width : 1px; height : ".$box_height."px; width : ".$box_width."px;'>
			<div style='left : ".$slidemeter_left."px; position : relative; top : ".$slidemeter_top."px; text-align: left;'><input type='hidden' id='actualPic$id' value='0'/><input type='hidden' id='path$id' value='".muds_plugin_url('/img/simpleslide/')."' />
			<table width='100' style='background : url(".muds_plugin_url('/img/simpleslide/background.png').");'>
			<tr>
			<td width='25' align='center'>
			<input type='image' id='backslide$id' src='".muds_plugin_url('/img/simpleslide/leftarrow.png')."' onclick='doPrevious($id,slideImages$id);' />
			</td>
			<td align='center'>$throbber</td>
			<td width='25' align='center'>
			<input type='image' id='nextslide$id' src='".muds_plugin_url('/img/simpleslide/rightarrow.png')."' onclick='doNext($id,slideImages$id);' />
			</td>
			</tr>
			</table>
			</div>
			<script type='text/javascript'>
			var slideImages$id = new Array(";
		//Create the array of photos
		$first=true;
		$first_image="";
		foreach($photo_gallery->photo as $photo) {
			//Get the photo 
			$media = str_replace('%size%', $size, $photo->resize);
			
			if(!$first) {
				$answer.=",";
			} else {
				$first=false;
				$first_image=$media;
			}
			$answer.="'".$media."'";
		}
		$answer.=")
			var actualPic=document.getElementById('actualPic$id').value;
			var div=document.getElementById('simple_gallery$id');
			div.style.backgroundImage='url(".$first_image.")';
			arrows($id,slideImages$id);
			changeText($id,slideImages$id);
			preloadImages(slideImages$id);
			</script>
			</div>";
		if($float=='center') $float="";
		
		//Ok, we have owr frame!!!!
		$answer="<div class=\"slide$float\"".$center.">$answer</div>";
	}
	
	return $answer;

}

/**
* Create the HTML code for a simple photo
*
* @access public
* @param xml data The data description of the gallery.
* @param int p_id The position of the photo in the gallery. Can be also LAST_PHOTO or RANDOM_PHOTO.
* @param int size The size of the photo in the post.
* @param string float Where the photo has to float.
* @param int conf A numeric value for the configuration.
* @return string The HTML code for the gallery the photos, false if not. 1 to show the first comment
	in picasaweb as a big description. 
*/
function muds_photo($data, $p_id, $size, $float, $conf) {
	
	$answer = false;
	if($photo_gallery = new SimpleXMLElement($data)) { //Parse the data
		if($p_id==RANDOM_PHOTO) $p_id=mt_rand(0,count($photo_gallery->photo)-1); // Show a random photo?
		if($p_id==LAST_PHOTO) $p_id=count($photo_gallery->photo)-1; //Show the last photo?
		if(count($photo_gallery->photo)>=$p_id) { //Do the gallery have a photo in this position?
			$photo=$photo_gallery->photo[$p_id-1]; //Ok, get the photo
			$answer="";
			
			//Get the data
			$media = str_replace('%size%', 800, $photo->resize);
			$thumbnail = str_replace('%size%', $size, $photo->resize);
			$title = $photo->title;
			$caption=false;
			if($conf & 1) { //Should we show the comment?
				$caption = $photo->comment;
			}
			$answer=muds_show($media, $thumbnail, $title, mt_rand(111111,999999), $float, $caption, $size);
				
		} else { //You are asking for something that is not here.
			$answer=__('Photo doesn\'t exist', 'mudslide' );
		}
	}	
	return $answer;
}

/**
* Enable buttons in tinymce.
* This function should be called by an action.
*
* @access public
*/
function muds_add_buttons() {
	// Don't bother doing this stuff if the current user lacks permissions
	if ( !current_user_can('edit_posts') && !current_user_can('edit_pages') ) return;

	// Add only in Rich Editor mode
	if ( get_user_option('rich_editing') == 'true') {
	 
		// add the button for wp21 in a new way
		add_filter('mce_external_plugins', 'add_mudslide_script');
		add_filter('mce_buttons', 'add_mudslide_button');
	}
}

/**
* Function to add the button to the bar.
* This function should be called by a filter.
*
* @access public
*/
function add_mudslide_button($buttons) {

	array_push($buttons, 'MudSlide');
	return $buttons;

}

/**
* Function to set the script which should answer when the user press the button.
* This function should be called by a filter.
*
* @access public
*/
function add_mudslide_script($plugins) {
	$pluginURL = muds_plugin_url('/tinymce/editor_plugin.js');
	$plugins['MudSlide'] = $pluginURL;
	return $plugins;
}

/**
* Return the HTML code for the photo in the widget.
*
* @access public
* @param int gallery The id of the gallery in the internal list.
* @param int photo_id The position of the photo in the gallery.
* @return string The HTML code. Empty if error. 
*/
function muds_widget_photo($gallery, $photo_id) {
	$type = muds_get_gallery_type($gallery);
	$answer="";
	switch($type) {
		case 2: //PicasaWeb
			$answer = mudsPicasa_widget_photo(muds_get_gallery_file($gallery), $photo_id);
			break;
		default:
			$answer=__('Unsupoerted source type', 'mudslide');
			break;
	}
	return $answer;
}

/**
* Return the HTML for the widget, using the configuration variables.
* If minimax isn't installed, this function returns a warning.
*
* @access public
* @return string The HTML code. Empty if error. 
*/
function muds_widget_contents( ) {
	global $wpdb;
	$options = get_option('widget_mudslide');
	$table_name = $wpdb->prefix . "mudslide";
	$answer=__('Album doesn\'t exist', 'mudslide' );
	if(function_exists('minimax_version') && minimax_version()>=MUDS_MNMX_V) { //Do we have minimax?
		switch($options['type']) {
			case MUDS_WIDGET_RANDOM:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY RAND() LIMIT 1"))
					$answer=muds_widget_photo($gallery_id,RANDOM_PHOTO);
				break;
			case MUDS_WIDGET_LAST:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY id DESC LIMIT 1"))
						$answer=muds_widget_photo($gallery_id,LAST_PHOTO);
				break;
			case MUDS_WIDGET_RANDOMLAST:
				if($gallery_id=$wpdb->get_var("SELECT id FROM $table_name ORDER BY id DESC LIMIT 1"))
						$answer=muds_widget_photo($gallery_id,RANDOM_PHOTO);
				break;
			case MUDS_WIDGET_RANDOMFROM:
				if(muds_get_gallery_file($options['gallery_id']))
					$answer=muds_widget_photo($options['gallery_id'],RANDOM_PHOTO);
				break;
		}
	} else { //We need minimax.
		$answer = sprintf(__('You have to install <a href="%s" target="_BLANK">minimax %1.1f</a> in order for this plugin to work.', 'mudslide'), "http://wordpress.org/extend/plugins/minimax/", MUDS_MNMX_V);
	}
	return $answer;
}

// mudslide widget stuff
function muds_widget_init() {

	if ( !function_exists('register_sidebar_widget') )
		return;

	function muds_widget($args) {

		// $args is an array of strings that help widgets to conform to
		// the active theme: before_widget, before_title, after_widget,
		// and after_title are the array keys. Default tags: li and h2.
		extract($args);
		
		// Each widget can store its own options. We keep strings here.
		$options = get_option('widget_mudslide');
		$title = $options['title'];
		$content=muds_widget_contents();
		
		// These lines generate our output. Widgets can be very complex
		// but as you can see here, they can also be very, very simple.
		echo $before_widget . $before_title . $title . $after_title . $content . $after_widget;
	}
	
	// This is the function that outputs the form to let the users edit
	// the widget's title. It's an optional feature that users cry for.
	function muds_widget_control() {
	
		// Get our options and see if we're handling a form submission.
		$options = get_option('widget_mudslide');
		if ( !is_array($options) ) {
			$options = array('title'=>'', 'type'=>'1', 'gallery_id' => '0');
		}
		if ( $_POST['mudslide-widget-submit'] ) {
			// Remember to sanitize and format use input appropriately.
			$options['title'] = strip_tags(stripslashes($_POST['mudslide_widget_title']));
			$options['type'] = $_POST['mudslide_widget_type'];
			if($options['type']==MUDS_WIDGET_RANDOMFROM) {
				$options['gallery_id']=$_POST['gallery_id'];
			} else {
			$options['gallery_id']=0;
			}
			update_option('widget_mudslide', $options);
		}

		// Be sure you format your options to be valid HTML attributes.
		$title = htmlspecialchars($options['title'], ENT_QUOTES);
		$type = $options['type'];
		$gallery_id=$options['gallery_id'];
		
		// Here is our little form segment. Notice that we don't need a
		// complete form. This will be embedded into the existing form.
		include('templates/mudslide_widget_manage.php');
	}
	
	// This registers our widget so it appears with the other available
	// widgets and can be dragged and dropped into any active sidebars.
	register_sidebar_widget(array('Mudslide Widget', 'widgets'), 'muds_widget');
	
	// This registers our optional widget control form. Because of this
	// our widget will have a button that reveals a 300x100 pixel form.
	register_widget_control(array('Mudslide Widget', 'widgets'), 'muds_widget_control');
}

// Run our code later in case this loads prior to any required plugins.
add_action('widgets_init', 'muds_widget_init');

?>
