<?php
/* Copyright 2007-2009 Juan Sebastián Echeverry (email : sebaxtian@gawab.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
*/

define('MUDS_LOCAL', 1);
define('MUDS_PICASA', 2);
define('MUDS_HIGH', 100000000);
define('MUDS_PICASA_XML_V', 2.0);

/**
* Returns the last used username in the editor.
*
* @access public
* @return string The last used username in the editor. 
*/
function mudsPicasa_get_user() {
	global $muds_user;
	return $muds_user;
}

/**
* Sets the user for future references.
*
* @access public
* @param string The username. 
*/
function mudsPicasa_set_user($new_user) {
	global $muds_user;
	update_option('muds_user', $new_user);
	$muds_user=$new_user;
}

/**
* Returns the list of galleries a user has.
*
* @access public
* @param string user The username in picasa.
* @return array The list of galleries. 
*/
function mudsPicasa_list_user_galleries($user) {
	//The default answer is an empty array, it means we can't read the file, nor create it. 
	$answer = array();
	
	//Ask for the user file 
	$url=muds_plugin_cache("pl-$user.xml");
	if(!mnmx_readfile($url)) { //If the user doesn't have a file yet
		mudsPicasa_update_user_galleries_list($user); //try to create it.
	}
	
	
	if($data = mnmx_readfile($url)) { //Try again to read the user file, maybe we have created it yet
		$data = new SimpleXMLElement($data); //Parse the XML file
		foreach($data->album as $album) { //Read each album
			$id = $album->id;
			$title = $album->title;
			array_push($answer, array('id'=>$id, 'title'=>$title)); //Fill the array with the data we need
		}
	}
	return $answer;
}

/**
* Updates the list of galleries a user has.
*
* @access public
* @param string user The username in picasa.
* @return bool True on succes, false if not. 
*/
function mudsPicasa_update_user_galleries_list($user) {
	//The default answer
	$answer = false;
	
	//A flag to know if we can save the file
	$save = false;
	
	//Start the XML file for the user's galleries.
	$out="<?xml version = '1.0' encoding = 'UTF-8'?><slide version='".MUDS_PICASA_XML_V."' owner='$user'>";
	
	//Hello Picasa, we have a little question. Do you know this user? Can you send the
	//albums he has?
	$url = "http://picasaweb.google.com/data/feed/api/user/$user?max-results=".MUDS_HIGH;
	if($data = mnmx_readfile($url)) { //Ask picasa
		if($data[0]=='<') { //Ok, we have an answer and it looks like an XML
			$save = true; //We are reading something, so we can save something
			$data = new SimpleXMLElement($data); //Parse the XML
			foreach($data->entry as $entry) { //Read each entry
				$id = $entry->xpath("gphoto:id"); //Get the id
				$id = $id[0];
				
				$title = $entry->title; //Get the title
				$title=str_replace("&", "&amp;", $title); //To save it correctly in the file and in MySQL
				
				$out.="<album><id>$id</id><title>$title</title></album>"; //Add the entry to the XML file
			}
					
			//Hello Picasa, we again. Can you send the tags this user has?
			$url = "http://picasaweb.google.com/data/feed/api/user/$user?kind=tag&max-results=".MUDS_HIGH;
			if($data = mnmx_readfile($url)) { //Ask picasa
				$data = new SimpleXMLElement($data); //Parse the XML
				foreach($data->entry as $entry) { //Read each entry
					$tag = $entry->summary; //Get the tag
					$tag_id = (string)$tag;
					if(is_numeric($tag_id)) { //A numeric tag can be a problem, rename it
						$tag_id = "tag#$tag#"; //Pray no one would use this kind of tags 
					}
					$out.="<album><id>$tag_id</id><title>".sprintf(__('Tag: %s', 'mudslide'), $tag)."</title></album>"; //Add the entry to the XML file
				}
			}
		}
		
		$out.="</slide>";
		
		//Saves the file
		if($save) {	
			$answer = false;
			if( $fwr = @fopen(muds_plugin_cache("pl-$user.xml"), "w")) {
				fwrite($fwr,$out);
				fclose($fwr);
				$answer=true; //Yes, we did it
			}
		}
	}
	
	return $answer;
}

/**
* Does an album existsts in the internal list?.
*
* @access public
* @param int album The album code to search.
* @return bool True on succes, false if not. 
*/
function mudsPicasa_album_exists( $album ) {
	global $wpdb;
	$table_name = $wpdb->prefix . "mudslide";
	$answer=false;
	if($wpdb->get_var("SELECT id FROM $table_name WHERE file='pa-$album.xml'")) {
		$answer=true;
	}
	return $answer;
}

/**
* Does an tag album existsts in the internal list?.
*
* @access public
* @param int album The album code to search.
* @return bool True on succes, false if not. 
*/
function mudsPicasa_tag_exists( $tag ) {
	global $wpdb;
	
	if(is_numeric($tag)) { //A numeric tag can be a problem, rename it
		$tag = "tag#$tag#"; //Pray no one would use this kind of tags 
	}
	
	$table_name = $wpdb->prefix . "mudslide";
	$answer=false;
	if($wpdb->get_var("SELECT id FROM $table_name WHERE file='pt-$tag.xml'")) {
		$answer=true;
	}
	return $answer;
}

/**
* Check if a file is old or doesn't exists
*
* @access public
* @param string filename The file.
* @return bool True if it's old or doesn't exists, false if exists and is the version we need. 
*/
function mudsPicasa_not_readable_file( $filename )
{
	$answer = false;
	$url = muds_plugin_cache($filename);
	if(!$data = mnmx_readfile($url)) { //Doesn't exist
		$answer = true; 
	} else { //Exists
		$data = new SimpleXMLElement($data); //Pase XML
		$version = $data->attributes()->version;//Get version
		if($version<MUDS_PICASA_XML_V) { //Is older
			$answer = true;
		}
	}
	return $answer;
}

/**
* Return the list of photos in a gallery.
*
* @access public
* @param string user The username in picasa.
* @param string gallery The id or tag from the gallery where we have to get the photos.
* @return array The list of photos, false if not. 
*/
function mudsPicasa_list_photos_in_gallery( $user, $gallery ) {
	$answer = array();
	$url = false;
	if(!is_numeric($gallery)) { //If this is a tag
		$filename = "pt-$gallery.xml";
		$url = muds_plugin_cache($filename); //Get the album file
		if(mudsPicasa_not_readable_file($filename)) { //Do we have this album file?
			mudsPicasa_update_tag($user, $gallery); //No! Try to create it
		}
	} else { //If it's an album
		$filename = "pa-$gallery.xml";
		$url = muds_plugin_cache($filename); //Get the album file
		if(mudsPicasa_not_readable_file($filename)) { //Do we have this album file?
			mudsPicasa_update_album($user, $gallery); //No! Try to create it
		}
	}
	
	if($url) { //So, do we have a file?
		if($data = mnmx_readfile($url)) { //Ok, try again to read it.
			$data = new SimpleXMLElement($data); //Parse the XML
			$id=1;
			foreach($data->photo as $photo) { //Read each entry
				$title = $photo->title;
				array_push($answer, array('id'=>$id, 'title'=>$title)); //Add the photo to the array
				$id++;
			}
		}
	}
	
	return $answer;
}

/**
* Updates a gallery.
*
* @access public
* @param string user The username in picasa.
* @param int album The id of the album from where we have to get the photos.
* @return array The list of photos, false if not. 
*/
function mudsPicasa_update_gallery( $user, $gallery ) {
	$answer = false;
	
	if(!is_numeric($gallery)) { //If it's a tag
		$answer = mudsPicasa_update_tag( $user, $gallery ); //Call the function to update the tag gallery
	} else {
		if($gallery>0) { //If it's an album
			$answer = mudsPicasa_update_album( $user, $gallery ); //Call the function to update the album
		}
	}
	return $answer;
}

/**
* Updates a gallery using the filename.
*
* @access public
* @param string filename The file to check.
* @return bool
*/
function mudsPicasa_update_gallery_by_filename( $filename ) {
	$answer = false;
	$url = muds_plugin_cache($filename);
	if($data = mnmx_readfile($url)) { //Does this file exists?
		$photo_gallery = new SimpleXMLElement($data); //Parse the XML
		//Get the data
		$owner = $photo_gallery->attributes()->owner;
		$gallery = $photo_gallery->attributes()->id;
		$tag = $photo_gallery->attributes()->tag;
			
		if(strlen($gallery)>0) { //If we found a gallery
			$answer = mudsPicasa_update_album($owner, $gallery);
		} else { //We found a tag gallery
			$answer = mudsPicasa_update_tag($owner, $tag);
		}
	}
	return $answer;
}

/**
* Updates an album.
*
* @access public
* @param string user The username in picasa.
* @param int album The id of the album from where we have to get the photos.
* @return array The list of photos, false if not. 
*/
function mudsPicasa_update_album( $user, $album ) {
	$fwr = false;
	if($album>0) { //If it is a number
		//Hello picasa, we again. Can you show the photos from this album?
		$url = "http://picasaweb.google.com/data/feed/api/user/$user/albumid/$album?max-results=".MUDS_HIGH;
		$gallery_title=__('Album doesn\'t exist', 'mudslide' ); //Maybe we are wrong, so this is to create a dummy answer
		if($data = mnmx_readfile($url)) { //Ask picasa
			$data = new SimpleXMLElement($data); //Parse the XML
			$gallery_title=$data->title; //Get the title
			$gallery_title=str_replace("&", "&amp;", $gallery_title); //To save it correctly in the file and in MySQL
			//Creates the XML data
			$out="<?xml version = '1.0' encoding = 'UTF-8'?><slide version='2.0' description='$gallery_title' owner='$user' id='$album' >"; //Our XML header
			foreach($data->entry as $entry) {
				$title=$entry->summary; //Get the title
				$title=str_replace("&", "&amp;", $title); //To save it correctly in the file and in MySQL
				if(strlen($title)==0) $title = __("No name", 'mudslide'); //If there are not title, use a dummy title.
				
				$id = $entry->xpath("gphoto:id"); //Get the id from the photo
				$id = $id[0];
				
				//Get the source file
				$src = $entry->content->attributes()->src;
				
				//Get the url to resize a file
				$dirname=dirname($src);
				$basename=basename($src);
				$resize = "$dirname/s%size%/$basename";
				
				//Get the comment
				$comment="";
				//Hello picasa, we again. Can you send the first comment this photo have?
				$photo_uri="http://picasaweb.google.com/data/feed/api/user/$user/albumid/$album/photoid/$id?kind=comment&max-results=1";

				if($data_comment = mnmx_readfile($photo_uri)) { //Ask picasa
					$rss_comment = new SimpleXMLElement($data_comment); //Parse the XML
					$entry=$rss_comment->entry;
					//If there is a comment
					if($entry) {
						$comment=$entry->content;
					}
				}
				
				//Create the XML entry
				$out.="<photo album='$album' id='$id'><title>$title</title><comment>$comment</comment><src>$src</src><resize>$resize</resize></photo>";
			}
			$out.="</slide>"; //Close the XML file
		} else { //Oops, picasa didn't answer. Ok, we don't have something to tell 
			$out=false;
		}
		
		if($out) { //If we have something to save	
			$url = muds_plugin_cache("pa-$album.xml");
			if($fwr = @fopen($url, "w")) { //Try to write the tag album
				fwrite($fwr,$out);
				fclose($fwr);
				
				//Creates the album in database if it doesn't exist
				if(!mudsPicasa_album_exists( $album )) {
					global $wpdb;
					$table_name = $wpdb->prefix . "mudslide";
					$insert = "INSERT INTO " . $table_name .
						" (type, file, name) " .
						"VALUES ('".MUDS_PICASA."','pa-$album.xml','$gallery_title')";

					$results = $wpdb->query( $insert );
				}
			}
		}
	}
	
	return $fwr;
}

/**
* Updates a tag album.
*
* @access public
* @param string user The username in picasa.
* @param int album The id of the tag album from where we have to get the photos.
* @return array The list of photos, false if not. 
*/
function mudsPicasa_update_tag( $user, $tag ) {
	//We put some code with the numeric tags, we have to delete it to use here
	$search = "@tag#(\d+)#@i";
	if(preg_match_all($search, $tag, $matches)) {
		if (is_array($matches)) {
			foreach ($matches[1] as $key =>$v0) {
				$tag=$matches[1][$key]; //if we found something, use it
			}
		}
	}

	//Hello picasa, we again. Can you show the photos with this tag?
	$url = "http://picasaweb.google.com/data/feed/api/user/$user?kind=photo&tag=$tag&max-results=".MUDS_HIGH;
	$gallery_title=__('Album doesn\'t exist', 'mudslide' ); //Maybe we are wrong, so this is to create a dummy answer
	if($data = mnmx_readfile($url)) { //Ask picasa
		$data = new SimpleXMLElement($data); //Parse the XML
		$gallery_title=sprintf(__('Tag: %s', 'mudslide'), $tag); //This is our gallery title
		$gallery_title=str_replace("&", "&amp;", $gallery_title); //To save it correctly in the file and in MySQL
		//Start the XML
		$out="<?xml version = '1.0' encoding = 'UTF-8'?><tag version='2.0' description='$gallery_title' owner='$user' tag='$tag' >";
		$n=1;
		foreach($data->entry as $entry) {
			$title=$entry->summary; //Get the title
			$title=str_replace("&", "&amp;", $title); //To save it correctly in the file and in MySQL
			if(strlen($title)==0) $title = __("No name", 'mudslide'); //If there are not title, use a dummy title.
			
			$id = $entry->xpath("gphoto:id"); //Get the id from the photo
			$id = $id[0];
			
			$album = $entry->xpath("gphoto:albumid"); //Get the album id from the photo
			$album = $album[0];
			
			//Get the source file
			$src = $entry->content->attributes()->src;
			
			//Get the url to resize a file
			$dirname=dirname($src);
			$basename=basename($src);
			$resize = "$dirname/s%size%/$basename";
			
			//Create the XML entry
			$out.="<photo album='$album' id='$id'><title>$title</title><comment/><src>$src</src><resize>$resize</resize></photo>";
		}
		$out.="</tag>"; //Close the XML
	} else { //Oops, Picasa didn't answer. We don't have something to show
		$out=false;
	}
	
	if($out) { //Do we have something to show?
		$tag_id = (string)$tag;
		if(is_numeric($tag_id)) { //A numeric tag can be a problem, rename it
			$tag= "tag#$tag#"; //Pray no one would use this kind of tags 
		}
		if($fwr = @fopen(muds_plugin_cache("pt-$tag.xml"), "w")) { //Try to save the file
			fwrite($fwr,$out);
			fclose($fwr);
			
			//Creates the album in database if it doesn't exist
			if(!mudsPicasa_tag_exists( $tag )) {
				global $wpdb;
				$table_name = $wpdb->prefix . "mudslide";
				$insert = "INSERT INTO " . $table_name .
					" (type, file, name) " .
					"VALUES ('".MUDS_PICASA."','pt-$tag.xml','$gallery_title')";
				$results = $wpdb->query( $insert );
			}
		}
	}
	
	return $fwr;
}

/**
* Returns the HTML code of a gallery. If the user can modify the post this function
* also puts a link to update he gallery.
*
* @access public
* @param string user The username in picasa.
* @param string gallery The id of the gallery from where we have to get the photos.
* @param int first The first photo of the list to show.
* @param int last The last photo of the list to show.
* @param int conf A numeric value for the configuration. 1 to show the first comment
	in picasaweb as a big description.
* @return string The HTML code for the gallery the photos, false if not. 
*/
function mudsPicasa_show_gallery( $user, $gallery, $first=0, $last=0, $conf=0 ) {
	$answer = "";
	$filename = "pt-$gallery.xml"; //We suppose it's a tag
	if(is_numeric($gallery)) { //If the gallery is numeric, it means is an album
		$filename = "pa-$gallery.xml";
	}
	
	$url = muds_plugin_cache($filename); //Get the path to the file
	$update_post = "type=picasa&amp;user=$user&amp;gallery=$gallery"; //Create the update post

	if(!mudsPicasa_not_readable_file($filename)) { //If it's a readable gallery
		//We have data, so we can try to show the gallery.
		$answer = muds_gallery(mnmx_readfile($url), $update_post, $first=0, $last=0, $conf);
	} else { //Oops, something goes wrong, it is not a readable gallery
		if(mudsPicasa_update_gallery( $user, $gallery )) { //Try to create it
			$answer = mudsPicasa_show_gallery( $user, $gallery, $first, $last, $conf ); //If we created it, show the gallery
		} else { //Oops, we can't create it
			$answer=__('XML File doesn\'t exist', 'mudslide' );
		}
	}
	return $answer;
}

/**
* Returns the HTML code of a simple frame gallery. If the user can modify the post this function
* also puts a link to update he gallery.
*
* @access public
* @param string user The username in picasa.
* @param string gallery The id of the gallery from where we have to get the photos.
* @param int conf A numeric value for the configuration.
* @return string The HTML code for the gallery the photos, false if not. 
*/
function mudsPicasa_show_simple($user, $gallery, $size=200, $float=false, $conf=0) {
	$answer = "";
	$filename = "pt-$gallery.xml"; //We suppose it's a tag
	if(is_numeric($gallery)) { //If the gallery is numeric, it means is an album
		$filename = "pa-$gallery.xml";
	}
	
	$url = muds_plugin_cache($filename); //Get the path to the file
	$update_post = "type=picasa&amp;user=$user&amp;gallery=$gallery"; //Create the update post
	
	if(!mudsPicasa_not_readable_file($filename)) { //If it's a readable file
		//We have data, so we can try to show the simple album.
		$answer = muds_simple(mnmx_readfile($url), $update_post ,$size, $float, $conf);
	} else { //Oops, something goes wrong, it is not a readable gallery.
		if(mudsPicasa_update_gallery( $user, $gallery )) { //Try to create it
			$answer = mudsPicasa_show_simple($user, $gallery, $size, $float, $conf); //If we created it, show the gallery
		} else { //Oops, we can't create it
			$answer=__('XML File doesn\'t exist', 'mudslide' );
		}
	}
	return $answer;
}

/**
* Returns the HTML code of a single photo.
*
* @access public
* @param string user The username in picasa. Can be NO_USER
* @param string gallery The id of the gallery from where we have to get the photo.
* @param int p_id The position of the photo in the gallery. Can be also LAST_PHOTO or RANDOM_PHOTO.
* @param int size The size of the photo in the post.
* @param string float Where the photo has to float.
* @param int conf A numeric value for the configuration.
* @return string The HTML code for the gallery the photos, false if not. 1 to show the first comment
	in picasaweb as a big description. 
*/
function mudsPicasa_show_photo( $user, $gallery, $p_id, $size=200, $float=false, $conf=0 ) {
	$answer = "";
	$filename = "pt-$gallery.xml"; //We suppose it's a tag
	if(is_numeric($gallery)) { //If the gallery is numeric, it means is an album
		$filename = "pa-$gallery.xml";
	}
	$url = muds_plugin_cache($filename); //Get the path to the file

	if(!mudsPicasa_not_readable_file($filename)) { //If it's a readable file
		//We have data, so we can try to show the photo.
		$answer = muds_photo(mnmx_readfile($url), $p_id, $size, $float, $conf);
	} else { //Oops, something goes wrong, we don't have the file.
		//Try to create it if we have the user. The widget doesn't know it.
		if($user && mudsPicasa_update_gallery( $user, $gallery )) {
			$answer = mudsPicasa_show_photo( $user, $gallery, $p_id, $size, $float, $conf ); //If we created the gallery, show the photo
		} else { //Oops, we can't create the gallery
			$answer=__('XML File doesn\'t exist', 'mudslide' );
		}
	}
	
	return $answer;
}

/**
* Return the HTML code for the photo in the widget.
*
* @access public
* @param string filename The file of the album to select from.
* @param int p_id The position of the photo in the gallery.
* @return string The HTML code. Empty if error. 
*/
function mudsPicasa_widget_photo($filename, $p_id) {
	$answer = "";
	$url = muds_plugin_cache($filename); //Get the path to the file
	
	if(!mudsPicasa_not_readable_file($filename)) { //If it's a readable file
		//We have data, so we can try to show the photo.
		$answer = muds_photo(mnmx_readfile($url), $p_id, 200, 'center', 0);
	} else { 
		//Try to update the gallery.
		if(mudsPicasa_update_gallery_by_filename( $filename )) { //If we can update the file
			//We have data, so we can try to show the photo.
			$answer = muds_photo(mnmx_readfile($url), $p_id, 200, 'center', 0);
		} else { 
			$answer = __('XML File doesn\'t exist', 'mudslide' );
		}
	}
	
	return $answer;
}

?>
